﻿namespace HeuristicLab.Problems.ProgramSynthesis.Base.Erc {
  using System;

  using HeuristicLab.Common;
  using HeuristicLab.Core;
  using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
  using HeuristicLab.Problems.ProgramSynthesis.Base.Weighted;

  [StorableClass]
  public abstract class ErcOptionConvertible<TValue, TValueItem> : ItemCollection<IWeightedErcValueItem<TValueItem>>, IErcValueItem<TValue> {
    private readonly Converter<TValueItem, TValue> converter;

    protected ErcOptionConvertible(Converter<TValueItem, TValue> converter) : this(converter, false) { }

    protected ErcOptionConvertible(Converter<TValueItem, TValue> converter, bool isEnabled, params IWeightedErcValueItem<TValueItem>[] values) {
      this.converter = converter;
      IsEnabled = isEnabled;
      AddRange(values);
    }

    [StorableConstructor]
    protected ErcOptionConvertible(bool deserializing)
      : base(deserializing) {
    }

    protected ErcOptionConvertible(ErcOptionConvertible<TValue, TValueItem> origin, Cloner cloner)
      : base(origin, cloner) {
    }

    public bool IsEnabled { get; set; }


    public TValue GetErcValue(IRandom random) {
      if (Count == 0 || !IsEnabled) return default(TValue);

      var ercValueItem = list.RandomWeightedOrDefault(random);
      return ercValueItem == null
        ? default(TValue)
        : converter(ercValueItem.GetErcValue(random));
    }
  }
}
