﻿namespace HeuristicLab.Problems.ProgramSynthesis.Base.Erc.Integer {
  using HeuristicLab.Common;
  using HeuristicLab.Core;
  using HeuristicLab.Data;
  using HeuristicLab.Parameters;
  using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

  [StorableClass]
  public class IntegerRangeErcValue : WeightedErcValueItem<int> {
    private const string RangeParameterName = "Range";

    public IntegerRangeErcValue() {
      Name = "Integer range";
      Parameters.Add(new FixedValueParameter<IntRange>(RangeParameterName));
    }

    public IntegerRangeErcValue(int start, int end) : this() {
      Start = start;
      End = end;
    }

    [StorableConstructor]
    public IntegerRangeErcValue(bool deseralizing) : base(deseralizing) { }

    public IntegerRangeErcValue(IntegerRangeErcValue origin, Cloner cloner) : base(origin, cloner) { }

    public IValueParameter<IntRange> RangeParameter
    {
      get { return (IValueParameter<IntRange>)this.Parameters[RangeParameterName]; }
    }

    public int Start
    {
      get { return this.RangeParameter.Value.Start; }
      set { this.RangeParameter.Value.Start = value; }
    }

    public int End
    {
      get { return this.RangeParameter.Value.End; }
      set { this.RangeParameter.Value.End = value; }
    }

    public override int GetErcValue(IRandom random) {

      return this.Start < this.End ? random.Next(this.Start, this.End) : 0;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new IntegerRangeErcValue(this, cloner);
    }
  }
}
