﻿using System.Collections.Generic;
using System.Linq;

namespace HeuristicLab.Problems.ProgramSynthesis.Base.Weighted {
  using HeuristicLab.Core;
  using HeuristicLab.Random;

  public static class WeightedExtensions {
    public static T RandomWeightedOrDefault<T>(this IEnumerable<T> items, IRandom random = null) where T : IWeighted {
      random = random ?? new MersenneTwister();
      var totalWeight = items.Sum(x => x.Weight);
      // The weight we are after...
      var itemWeightIndex = random.NextDouble() * totalWeight;
      var currentWeightIndex = 0d;

      foreach (var item in items) {
        currentWeightIndex += item.Weight;

        // If we've hit or passed the weight we are after for this item then it's the one we want....
        if (currentWeightIndex >= itemWeightIndex)
          return item;
      }

      return default(T);
    }
  }
}
