﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using System.Collections.Generic;

  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  /// <summary>
  /// Conj's an item onto the type stack.
  /// </summary>
  public abstract class VectorConjExpression<T> : StatelessExpression {
    protected bool Eval(IInternalPushInterpreter interpreter, IPushStack<List<T>> vectorStack, IPushStack<T> literalStack) {
      if (vectorStack.IsEmpty ||
          literalStack.IsEmpty ||
          vectorStack.Top.Count + 1 > interpreter.Configuration.MaxVectorLength)
        return false;

      var literal = literalStack.Pop();
      vectorStack.Top.Add(literal);
      return true;
    }
  }

  [PushExpression(StackTypes.IntegerVector, "INTEGER[].CONJ", StackTypes.Integer)]
  public class IntegerVectorConjExpression : VectorConjExpression<long> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.IntegerVectorStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackTypes.FloatVector, "FLOAT[].CONJ", StackTypes.Float)]
  public class FloatVectorConjExpression : VectorConjExpression<double> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.FloatVectorStack, interpreter.FloatStack);
    }
  }

  [PushExpression(StackTypes.StringVector, "STRING[].CONJ", StackTypes.String)]
  public class StringVectorConjExpression : VectorConjExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.StringVectorStack, interpreter.StringStack);
    }
  }

  [PushExpression(StackTypes.BooleanVector, "BOOLEAN[].CONJ", StackTypes.Boolean)]
  public class BooleanVectorConjExpression : VectorConjExpression<bool> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter, interpreter.BooleanVectorStack, interpreter.BooleanStack);
    }
  }
}